==> fred-ferda-3.13.2.tar.gz: fred-ferda-3.13.2/ferda/registry/domain/api.py <==
"""Registry domain REST API views."""
import asyncio
import json
import logging
from datetime import datetime, timedelta
from typing import Any, Callable, Dict, List, Optional, Set, cast

import pytz
from asgiref.sync import sync_to_async
from grill import LogProperty
from ninja import Query, Router, Schema
from regal import ContactId, Domain, DomainHistoryId, DomainId
from regal.common import DatetimeRange
from regal.domain import ContactInfo, ContactInfoType, DomainContactRole
from regal.exceptions import ContactDoesNotExist, DomainDoesNotExist

from ferda.api import Error, TokenPaginatedList, check_permissions
from ferda.backend import DOMAIN, DOMAIN_ADMIN, KEYSET, LOGGER, NSSET
from ferda.constants import LOGGER_SESSION_KEY, LogEntryType, LoggerResult, LogReferenceType, StateFlag
from ferda.settings import SETTINGS

from .schema import DomainByContact, DomainContactInfo, UpdateDomainAdditionalContacts

_LOGGER = logging.getLogger(__name__)

router = Router(tags=['registry', 'domain'])


async def _get_domain_by_contact_info(domain: Dict[str, Any], semaphore: asyncio.Semaphore) -> Dict[str, Any]:
    """Get dictionary with domain info and roles of contact in relation to that domain."""
    async with semaphore:
        domain_info = (await DOMAIN.get_domain_info(
            cast(DomainId, domain.get('domain_id')),
            cast(DomainHistoryId, domain.get('domain_history_id')),
        )).dict()

        # XXX: This is a temporary fix until related object handles are added to the API
        if keyset := domain_info.get('keyset'):
            domain_info['keyset_handle'] = (await KEYSET.get_keyset_info(keyset)).keyset_handle
        if nsset := domain_info.get('nsset'):
            domain_info['nsset_handle'] = (await NSSET.get_nsset_info(nsset)).nsset_handle

    domain_info.pop('auth_info')  # Auth info is only accessible through separate protected API.
    return {
        'domain': domain_info,
        'roles': set(cast(set, domain.get('roles'))),
    }


class DomainsByContactQuery(Schema):
    """Get domains by contact query."""

    roles: Set[DomainContactRole]
    order_by: Optional[str] = None
    fqdn_filter: Optional[str] = None
    page_token: Optional[str] = None
    page_size: Optional[int] = None


@router.get('/by_contact/{contact_id}/', url_name='get_domains_by_contact',
            response={200: TokenPaginatedList[DomainByContact], 400: Error, 404: Error})
async def get_domains_by_contact(request, contact_id: ContactId, query: DomainsByContactQuery = Query(None)):
    """Get domains by contact."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])

    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_LIST,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={
            'roles': [role for role in DomainContactRole if role in query.roles],  # Maintain stable ordering
            'order_by': query.order_by or '',
            'fqdn_filter': query.fqdn_filter or '',
            'page_token': query.page_token or '',
            'page_size': '' if query.page_size is None else str(query.page_size),
        },
        references={LogReferenceType.CONTACT: contact_id},
    )
    with log_entry:
        try:
            result = await DOMAIN.get_domains_by_contact(
                contact_id, query.roles,
                order_by=query.order_by, fqdn_filter=query.fqdn_filter,
                page_token=query.page_token, page_size=query.page_size,
            )
            semaphore = asyncio.Semaphore(SETTINGS.concurrent_grpc_calls_limit)
            result['results'] = await asyncio.gather(
                *[_get_domain_by_contact_info(domain, semaphore=semaphore) for domain in result.pop('domains', [])],
            )
            log_entry.result = LoggerResult.SUCCESS
            return result
        except ContactDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Contact '%s' does not exist" % contact_id)
        except ValueError:
            log_entry.result = LoggerResult.FAIL
            return 400, Error(detail="invalid value")


@router.post('/set_manual_in_zone/{domain_id}/', url_name='set_domain_manual_in_zone', response={200: None, 404: Error})
async def set_domain_manual_in_zone(request, domain_id: DomainId, start: datetime = None, end: datetime = None):
    """Set domain manual in zone state flag."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_set_manual_in_zone'])

    if start is None:
        start = datetime.now(tz=pytz.utc)
    if end is None:
        end = start + timedelta(seconds=SETTINGS.manual_in_zone_default_duration)

    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_STATE_CHANGE,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={
            StateFlag.SERVER_INZONE_MANUAL: LogProperty('', children={
                'start': start.isoformat(),
                'end': end.isoformat(),
            })
        },
        references={LogReferenceType.DOMAIN: domain_id},
    )
    with log_entry:
        try:
            result = DOMAIN_ADMIN.manage_domain_state_flags(
                domains=[Domain(domain_id=domain_id, fqdn="")],
                state_flags={StateFlag.SERVER_INZONE_MANUAL: DatetimeRange(start=start, end=end)},
            )
            # We don't care about the result here,
            # but we still need to exhaust async generator
            async for _ in result:
                pass
            log_entry.result = LoggerResult.SUCCESS
        except DomainDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Domain '%s' does not exist" % domain_id)


async def get_additional_contacts(
    request, *, method: Callable, event: str, start: datetime, end: datetime,
    page_token: str = None, page_size: int = None,
) -> Dict[str, Any]:
    """Get additional domain notify contacts."""
    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_CONTACT_INFO,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={
            'event': event,
            'start': start.isoformat(),
            'end': end.isoformat(),
            'page_token': page_token or '',
            'page_size': str(page_size) if page_size is not None else ''
        },
    )
    with log_entry:
        result = await method(start, end, page_size=page_size, page_token=page_token)
        result['results'] = result.pop('domains')

        log_entry.result = LoggerResult.SUCCESS
        return result


@router.get('/get_additional_outzone_contacts/', url_name='get_additional_outzone_contacts',
            response=TokenPaginatedList[DomainContactInfo])
async def get_additional_outzone_contacts(
    request, start: datetime, end: datetime, page_token: str = None, page_size: int = None
):
    """Get additional domain outzone notify contacts."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])
    return await get_additional_contacts(
        request, method=DOMAIN_ADMIN.get_domains_outzone_notify_info, event='outzone',
        start=start, end=end, page_token=page_token, page_size=page_size,
    )


@router.get('/get_additional_delete_contacts/', url_name='get_additional_delete_contacts',
            response=TokenPaginatedList[DomainContactInfo])
async def get_additional_delete_contacts(
    request, start: datetime, end: datetime, page_token: str = None, page_size: int = None
):
    """Get additional domain delete notify contacts."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])
    return await get_additional_contacts(
        request, method=DOMAIN_ADMIN.get_domains_delete_notify_info, event='delete',
        start=start, end=end, page_token=page_token, page_size=page_size,
    )


async def update_additional_contacts(
    request, *, event: str, method: Callable, domains: Dict[DomainId, List[ContactInfo]],
    start: datetime, end: datetime,
):
    """Update additional notify contacts."""
    log_content = {
        domain_id: {
            info_type: [contact.value for contact in contacts if contact.type == info_type]
            for info_type in ContactInfoType
        } for domain_id, contacts in domains.items()
    }

    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_CONTACT_INFO_UPDATE,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={'event': event, 'start': start.isoformat(), 'end': end.isoformat()},
        references={LogReferenceType.DOMAIN: list(domains.keys())},
        content=json.dumps(log_content),
    )
    with log_entry:
        try:
            await method(domains, start=start, end=end)

            log_entry.result = LoggerResult.SUCCESS
            return 200, None
        except DomainDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Domain does not exist")
        except ValueError:
            log_entry.result = LoggerResult.FAIL
            return 400, Error(detail="Invalid value")


@router.post('/update_additional_outzone_contacts/', url_name='update_additional_outzone_contacts',
             response={200: None, 400: Error, 404: Error})
async def update_additional_outzone_contacts(request, body: UpdateDomainAdditionalContacts):
    """Update additional outzone notify contacts."""
    await sync_to_async(check_permissions)(
        request, permissions=['ferda.can_view_registry', 'ferda.can_update_additional_contacts'])
    return await update_additional_contacts(
        request, method=DOMAIN_ADMIN.update_domains_outzone_additional_notify_info,
        event='outzone', domains=body.domains, start=body.start, end=body.end,
    )


@router.post('/update_additional_delete_contacts/', url_name='update_additional_delete_contacts',
             response={200: None, 400: Error, 404: Error})
async def update_additional_delete_contacts(request, body: UpdateDomainAdditionalContacts):
    """Update additional delete notify contacts."""
    await sync_to_async(check_permissions)(
        request, permissions=['ferda.can_view_registry', 'ferda.can_update_additional_contacts'])
    return await update_additional_contacts(
        request, method=DOMAIN_ADMIN.update_domains_delete_additional_notify_info,
        event='delete', domains=body.domains, start=body.start, end=body.end,
    )

==> fred-ferda-3.15.0.tar.gz: fred-ferda-3.15.0/ferda/registry/domain/api.py <==
"""Registry domain REST API views."""
import asyncio
import json
import logging
from datetime import datetime, timedelta
from typing import Any, Callable, Dict, List, Optional, Set, cast

import pytz
from asgiref.sync import sync_to_async
from grill import LogProperty
from ninja import Query, Router, Schema
from regal import ContactId, Domain, DomainHistoryId, DomainId
from regal.common import DatetimeRange
from regal.domain import ContactInfo, ContactInfoType, DomainContactRole
from regal.exceptions import ContactDoesNotExist, DomainDoesNotExist

from ferda.api import Error, TokenPaginatedList, check_permissions
from ferda.backend import DOMAIN, DOMAIN_ADMIN, KEYSET, LOGGER, NSSET, STATEMENTOR
from ferda.constants import LOGGER_SESSION_KEY, LogEntryType, LoggerResult, LogReferenceType, StateFlag
from ferda.registry.record_statement import get_object_statement_pdf
from ferda.settings import SETTINGS

from .schema import DomainByContact, DomainContactInfo, UpdateDomainAdditionalContacts

_LOGGER = logging.getLogger(__name__)

router = Router(tags=['registry', 'domain'])


async def _get_domain_by_contact_info(domain: Dict[str, Any], semaphore: asyncio.Semaphore) -> Dict[str, Any]:
    """Get dictionary with domain info and roles of contact in relation to that domain."""
    async with semaphore:
        domain_info = (await DOMAIN.get_domain_info(
            cast(DomainId, domain.get('domain_id')),
            cast(DomainHistoryId, domain.get('domain_history_id')),
        )).dict()

        # XXX: This is a temporary fix until related object handles are added to the API
        if keyset := domain_info.get('keyset'):
            domain_info['keyset_handle'] = (await KEYSET.get_keyset_info(keyset)).keyset_handle
        if nsset := domain_info.get('nsset'):
            domain_info['nsset_handle'] = (await NSSET.get_nsset_info(nsset)).nsset_handle

    domain_info.pop('auth_info')  # Auth info is only accessible through separate protected API.
    return {
        'domain': domain_info,
        'roles': set(cast(set, domain.get('roles'))),
    }


class DomainsByContactQuery(Schema):
    """Get domains by contact query."""

    roles: Set[DomainContactRole]
    order_by: Optional[str] = None
    fqdn_filter: Optional[str] = None
    page_token: Optional[str] = None
    page_size: Optional[int] = None


@router.get('/record_statement/{domain_id}/', url_name='get_domain_record_statement_pdf', response={
    200: bytes,
    404: Error,
})
async def get_domain_record_statement_pdf(request, domain_id: DomainId):
    """Get verified record statement of the domain."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])
    return await get_object_statement_pdf(
        request, DomainId(domain_id), LogReferenceType.DOMAIN, STATEMENTOR.get_domain_statement
    )


@router.get('/by_contact/{contact_id}/', url_name='get_domains_by_contact',
            response={200: TokenPaginatedList[DomainByContact], 400: Error, 404: Error})
async def get_domains_by_contact(request, contact_id: ContactId, query: DomainsByContactQuery = Query(None)):
    """Get domains by contact."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])

    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_LIST,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={
            'roles': [role for role in DomainContactRole if role in query.roles],  # Maintain stable ordering
            'order_by': query.order_by or '',
            'fqdn_filter': query.fqdn_filter or '',
            'page_token': query.page_token or '',
            'page_size': '' if query.page_size is None else str(query.page_size),
        },
        references={LogReferenceType.CONTACT: contact_id},
    )
    with log_entry:
        try:
            result = await DOMAIN.get_domains_by_contact(
                contact_id, query.roles,
                order_by=query.order_by, fqdn_filter=query.fqdn_filter,
                page_token=query.page_token, page_size=query.page_size,
            )
            semaphore = asyncio.Semaphore(SETTINGS.concurrent_grpc_calls_limit)
            result['results'] = await asyncio.gather(
                *[_get_domain_by_contact_info(domain, semaphore=semaphore) for domain in result.pop('domains', [])],
            )
            log_entry.result = LoggerResult.SUCCESS
            return result
        except ContactDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Contact '%s' does not exist" % contact_id)
        except ValueError:
            log_entry.result = LoggerResult.FAIL
            return 400, Error(detail="invalid value")


@router.post('/set_manual_in_zone/{domain_id}/', url_name='set_domain_manual_in_zone', response={200: None, 404: Error})
async def set_domain_manual_in_zone(request, domain_id: DomainId, start: datetime = None, end: datetime = None):
    """Set domain manual in zone state flag."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_set_manual_in_zone'])

    if start is None:
        start = datetime.now(tz=pytz.utc)
    if end is None:
        end = start + timedelta(seconds=SETTINGS.manual_in_zone_default_duration)

    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_STATE_CHANGE,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={
            StateFlag.SERVER_INZONE_MANUAL: LogProperty('', children={
                'start': start.isoformat(),
                'end': end.isoformat(),
            })
        },
        references={LogReferenceType.DOMAIN: domain_id},
    )
    with log_entry:
        try:
            result = DOMAIN_ADMIN.manage_domain_state_flags(
                domains=[Domain(domain_id=domain_id, fqdn="")],
                state_flags={StateFlag.SERVER_INZONE_MANUAL: DatetimeRange(start=start, end=end)},
            )
            # We don't care about the result here,
            # but we still need to exhaust async generator
            async for _ in result:
                pass
            log_entry.result = LoggerResult.SUCCESS
        except DomainDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Domain '%s' does not exist" % domain_id)


async def get_additional_contacts(
    request, *, method: Callable, event: str, start: datetime, end: datetime,
    page_token: str = None, page_size: int = None,
) -> Dict[str, Any]:
    """Get additional domain notify contacts."""
    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_CONTACT_INFO,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={
            'event': event,
            'start': start.isoformat(),
            'end': end.isoformat(),
            'page_token': page_token or '',
            'page_size': str(page_size) if page_size is not None else ''
        },
    )
    with log_entry:
        result = await method(start, end, page_size=page_size, page_token=page_token)
        result['results'] = result.pop('domains')

        log_entry.result = LoggerResult.SUCCESS
        return result


@router.get('/get_additional_outzone_contacts/', url_name='get_additional_outzone_contacts',
            response=TokenPaginatedList[DomainContactInfo])
async def get_additional_outzone_contacts(
    request, start: datetime, end: datetime, page_token: str = None, page_size: int = None
):
    """Get additional domain outzone notify contacts."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])
    return await get_additional_contacts(
        request, method=DOMAIN_ADMIN.get_domains_outzone_notify_info, event='outzone',
        start=start, end=end, page_token=page_token, page_size=page_size,
    )


@router.get('/get_additional_delete_contacts/', url_name='get_additional_delete_contacts',
            response=TokenPaginatedList[DomainContactInfo])
async def get_additional_delete_contacts(
    request, start: datetime, end: datetime, page_token: str = None, page_size: int = None
):
    """Get additional domain delete notify contacts."""
    await sync_to_async(check_permissions)(request, permissions=['ferda.can_view_registry'])
    return await get_additional_contacts(
        request, method=DOMAIN_ADMIN.get_domains_delete_notify_info, event='delete',
        start=start, end=end, page_token=page_token, page_size=page_size,
    )


async def update_additional_contacts(
    request, *, event: str, method: Callable, domains: Dict[DomainId, List[ContactInfo]],
    start: datetime, end: datetime,
):
    """Update additional notify contacts."""
    log_content = {
        domain_id: {
            info_type: [contact.value for contact in contacts if contact.type == info_type]
            for info_type in ContactInfoType
        } for domain_id, contacts in domains.items()
    }

    log_entry = LOGGER.create(
        LogEntryType.DOMAIN_CONTACT_INFO_UPDATE,
        source_ip=request.META.get('REMOTE_ADDR', ''),
        session_id=request.session[LOGGER_SESSION_KEY],
        properties={'event': event, 'start': start.isoformat(), 'end': end.isoformat()},
        references={LogReferenceType.DOMAIN: list(domains.keys())},
        content=json.dumps(log_content),
    )
    with log_entry:
        try:
            await method(domains, start=start, end=end)

            log_entry.result = LoggerResult.SUCCESS
            return 200, None
        except DomainDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Domain does not exist")
        except ValueError:
            log_entry.result = LoggerResult.FAIL
            return 400, Error(detail="Invalid value")


@router.post('/update_additional_outzone_contacts/', url_name='update_additional_outzone_contacts',
             response={200: None, 400: Error, 404: Error})
async def update_additional_outzone_contacts(request, body: UpdateDomainAdditionalContacts):
    """Update additional outzone notify contacts."""
    await sync_to_async(check_permissions)(
        request, permissions=['ferda.can_view_registry', 'ferda.can_update_additional_contacts'])
    return await update_additional_contacts(
        request, method=DOMAIN_ADMIN.update_domains_outzone_additional_notify_info,
        event='outzone', domains=body.domains, start=body.start, end=body.end,
    )


@router.post('/update_additional_delete_contacts/', url_name='update_additional_delete_contacts',
             response={200: None, 400: Error, 404: Error})
async def update_additional_delete_contacts(request, body: UpdateDomainAdditionalContacts):
    """Update additional delete notify contacts."""
    await sync_to_async(check_permissions)(
        request, permissions=['ferda.can_view_registry', 'ferda.can_update_additional_contacts'])
    return await update_additional_contacts(
        request, method=DOMAIN_ADMIN.update_domains_delete_additional_notify_info,
        event='delete', domains=body.domains, start=body.start, end=body.end,
    )
